// PluginBusDevice.h: interface for the CPluginBusDevice class.
//
// Proxy class which passes calls on via a PlgComponent interface
// to the equivalent CPluginBusDevice of the given class.
//
//////////////////////////////////////////////////////////////////////
#if !defined(PROCESSORINTERFACE_H)
#define PROCESSORINTERFACE_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Interface.h"

// name for exported processor interfaces

#define IFACE_PROCESSOR "Processor"
#define IFACE_COPROCBUS "ArmCoprocBus"

struct IProcessor
{
	Interface iface;

	void (*SignalInterrupt)(IProcessor*, BOOL is_interrupt);
	void (*SignalFastInterrupt)(IProcessor*, BOOL is_interrupt);

	void (*Run)(IProcessor*);
	void (*Stop)(IProcessor*);
};

struct IArmCoproc;


// The Arm coprocessor interface is declared as type "ArmCoproc"
struct IArmCoproc
{
	Interface iface;

	Word opaque;		// private data for the coprocessor interface

	BOOL (*DataOp)(IArmCoproc*, Word instruction);
	BOOL (*RegisterTransOut)(IArmCoproc*, Word instruction, Word data);
	BOOL (*RegisterTransIn)(IArmCoproc*, Word instruction, Word* data);
	BOOL (*DataTransOut)(IArmCoproc*, Word instruction);
	BOOL (*DataTransIn)(IArmCoproc*, Word instruction);
};

// The Arm coprocessor bus is declared as "ArmCoprocBus"
struct IArmCoprocBus
{
	Interface iface;

	BOOL (*AddCoprocessor)(IArmCoprocBus*, IArmCoproc*, Byte number);
	BOOL (*RemoveCoprocessor)(IArmCoprocBus*, IArmCoproc*, Byte number);

	BOOL (*ReadWord)(IArmCoprocBus*, IArmCoproc*, Word* data);
	BOOL (*WriteWord)(IArmCoprocBus*, IArmCoproc*, Word data);
};

#endif // PROCESSORINTERFACE_H
