// SimpleDevice.h: interface for the CSimpleDevice class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SIMPLEDEVICE_H__ABA86EB0_A436_4F6E_BDAA_21C3EAADA111__INCLUDED_)
#define AFX_SIMPLEDEVICE_H__ABA86EB0_A436_4F6E_BDAA_21C3EAADA111__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Component.h"

#include "BusInterface.h"
#include "IocInterface.h"
#include "TimerInterface.h"

class CSimpleDevice : public CComponent  
{
protected:
	IPhysicalBusDevice idevice;		// This components bus interface
	IPhysicalBus* bus;				// handle to the bus
	IIoController* intc;		// Handle to the interrupt controller
	IScheduler* scheduler;			// Handle to the scheduler

	CSimpleDevice(Word base = 0, Word range = 0);

	Word map_base;		// base address in physical memory
	Word map_range;		// range in physical memory (including ghosting)

	Word map_basepage;	// base page in memory
	Word map_pagerange;	// number of pages mapped (including ghosting)

public:

	virtual ~CSimpleDevice();

	// CComponent interface
	virtual BOOL Init();
	virtual BOOL Connect();
	virtual BOOL Disconnect();

		// called when the arm writes to a memory location mapped to this
	// device. The functions may be NULL.
	virtual void WriteWord(Word page, Word offset, Word data);
	virtual void WriteHalf(Word page, Word offset, HalfWord data);
	virtual void WriteByte(Word page, Word offset, Byte data);

	// called when the arm reads from a memory location mapped to this
	// device. The functions may be NULL.
	virtual Word	 ReadWord(Word page, Word offset);
	virtual HalfWord ReadHalf(Word page, Word offset);
	virtual Byte	 ReadByte(Word page, Word offset);

	// If a page contains ram or rom rather than registers and
	// is effectively cacheable then returning a pointer to the
	// page via these functions will allow the arm cache to read
	// directly from the ram rather than calling the above functions.
	// If the functions are NULL or a NULL pointer is returned the
	// page will not be cached and the above read/write functions will
	// always be called
	virtual Byte* ProbeReadCacheable(Word page);
	virtual Byte* ProbeWriteCacheable(Word page);

	// These tell any bus masters to purge data stored in their caches
	// The actual effect is to force bus masters to call the read/write
	// functions rather than accessing any cached page pointers
	void PurgePageReadable(Word page);
	void PurgePageWriteable(Word page);

};

#endif // !defined(AFX_SIMPLEDEVICE_H__ABA86EB0_A436_4F6E_BDAA_21C3EAADA111__INCLUDED_)
